import time
import os

from sensor.Sensor import Sensor
from main import admin

from utils import i18n
from utils.datatypes import *

from docdb import docdb

from cal import cal

#from ev_editor import editor
from ev_list import events_list
from ev_viewer import event_viewer

class WeeklyCalendar(Sensor):
	def __init__(self, *args):
		global _; _ = i18n.Translator("calendar-sensor")
		global gfx; gfx = os.getcwd() + "/gfx/"
		
		self.width = 195
		
		# Set this if the user's browsing around months, so we don't
		# accidentally re-set self.page on a month rollover
		#self.navigating_months = 0
		self.orig_page = {}
	
	
		# call the constructor of the super class
		Sensor.__init__(self)

		# run a timeout function once a second
		
		self._set_config_type("first_weekday", TYPE_INT, 1)
		self._set_config_type("twentyfour_hour", TYPE_INT, 1)
		self._set_config_type("day_color", TYPE_STRING, "black")
		self._set_config_type("today_color", TYPE_STRING, "red")
		self._set_config_type("event_day_color", TYPE_STRING, "blue")
		self._set_config_type("event_color", TYPE_STRING, "black")
		self._set_config_type("dayname_font", TYPE_STRING, "Sans bold 10")
		self._set_config_type("dayname_color", TYPE_STRING, "black")
		self._set_config_type("day_font", TYPE_STRING, "Sans 10")
		self._set_config_type("bar1_color", TYPE_STRING, "#ff000088")
		self._set_config_type("bar2_color", TYPE_STRING, "black")

		self._add_timer(1000, self.create_display)
		self._add_timer(60000, self.update_display)
		self._add_timer(3600000, self.remove_earlier_events)

		self._watch_config(self.watch_config)
		
		self.event_views = {}
	
	def get_configurator(self):
	
		weekdays = [_("Sunday"), _("Monday")]
		tf = [_("12 Hour"), _("24 Hour")]
		
		cweekdays = []
		ctf = []
		i = 0
		for x in weekdays:
			cweekdays.append([x, i])
			ctf.append([tf[i], i])
			i += 1
		
		configurator = self._new_configurator()
		
		configurator.set_name(_("Weekly Calendar"))
		configurator.add_title(_("Weekly Calendar Settings"))
		configurator.add_option(_("First Weekday"), "first_weekday",
		                        _("The first day of the week."), cweekdays)
		configurator.add_option(_("Clock Format"), "twentyfour_hour",
								_("The format the times should be displayed in."), ctf)
		configurator.add_font_selector(_("Daynames font:"), "dayname_font",
		                               _("The font for the day names."))
		configurator.add_color_selector(_("Daynames color:"), "dayname_color",
		                                _("The color of the names of the days."))
		configurator.add_font_selector(_("Days font:"), "day_font",
		                               _("The font for the days."))
		configurator.add_color_selector(_("Day color:"), "day_color",
		                                _("The color of the calendar days."))
		configurator.add_color_selector(_("Today's color:"), "today_color",
		                                _("The color of today."))
		configurator.add_color_selector(_("Event day color:"), "event_day_color",
		                                _("The color of days with events."))
		configurator.add_color_selector(_("Event color:"), "event_color",
		                                _("The color of events on the task list."))
		configurator.add_color_selector(_("Bar color 1:"), "bar1_color",
		                                _("The color of the upper two horizontal bars."))
		configurator.add_color_selector(_("Bar color 2:"), "bar2_color",
		                                _("The color of the lowest horizontal bar."))
		
		return configurator
	
	def watch_config(self, key, value):
		self.create_display()
	
	def create_display(self):
	
		self.c = cal(self._get_config("first_weekday"))
		self.page = self.c.get_page(time.localtime())
		self.orig_page = self.page
	
		out = self._new_output()
		
		out.set("winwidth", self.width)
		out.set("wincenter", self.width / 2)
		for x in range(7):
			out.set("dayname_color[%d]"%x, self._get_config("dayname_color"))
			out.set("dayname_font[%d]"%x, self._get_config("dayname_font"))
		
		for x in range(42):
			out.set("day_font[%d]"%x, self._get_config("day_font"))
		
		self._send_output(out)
		
		self.remove_earlier_events()
		self.print_month()
		self.print_daynames()
		self.print_days()
		return 0
	
	def update_display(self):
		self.print_month()
		self.print_daynames()
		self.print_days()
		return 1
	
	def print_month(self):
		m,y = self.page
		if m != time.localtime()[1] and self.page == self.orig_page:
			self.page = self.c.page_increase(self.page)
			m,y = self.page
		out = self._new_output()
		out.set("month", self.c.get_monthname(m))
		out.set("monthx", self.width * .69)
		out.set("year", y)
		out.set("yearx", self.width * .71)
		self._send_output(out)
		return 1
		
	
	def print_daynames(self):
		output = self._new_output()
		for x in range(7):
			d = x + self._get_config("first_weekday")
			output.set("dayname[%d]"%x, self.c.next_dayname(d))
			output.set("main_dayname[%d]"%x, self.c.get_today_name(x))
		self._send_output(output)

	def print_days(self):
		out = self._new_output()
		stop = 0
		w, ld = self.c.get_fweek_mdays(self.page)
		today = self.c.get_today(self.page)
		tasks = self.c.get_days_with_tasks(self.page)
		orig_page = self.page

		d = today
		self.cur_date = {}
		self.cur_date_name = {}
		w += 1 - self._get_config("first_weekday")
		if w < 0:
			w += 7
		for i in range(7):			
			if d > ld:
				d = 1
				self.page = self.c.page_increase(self.page)
				tasks = self.c.get_days_with_tasks(self.page)
			if d <= 0 or stop == 1:
				out.set("main_day[%d]"%i, " ")
			else:
				out.set("main_day[%d]"%i, "%d"%d)
				self.cur_date[i] = self.page[1], self.page[0], d
			
			task_number_today = tasks.count(d)
			
			if d == today:
				out.set("main_dcol[%d]"%i, self._get_config("today_color"))
			elif task_number_today > 0:
				out.set("main_dcol[%d]"%i, self._get_config("event_day_color"))
			else:
				out.set("main_dcol[%d]"%i, self._get_config("day_color"))
			
			task_string = ""
			
			for cv in range(task_number_today):
				t1 = self.c.get_tasks_on_day(d, self.page)[cv]
				if t1.full_day == 1:
					task_string += "All day"
				else:
					if self._get_config("twentyfour_hour") == 1:
						st = t1.start
						en = t1.end
					else:
						st = self.c.twentyfour2twelve(t1.start)
						en = self.c.twentyfour2twelve(t1.end)
					
					if t1.start == t1.end:
						task_string += st
					else:
						task_string += st + " - " + en
				
				task_string += ": " + t1.summary + "\n"
			
			out.set("evcol[%d]"%i, self._get_config("event_color"))
			out.set("evs[%d]"%i, task_string)
			
			d += 1
		
		if self.page != orig_page:
			self.page = orig_page
			tasks = self.c.get_days_with_tasks(self.page)
		d = 0
		stop = 0
		
		w, ld = self.c.get_fweek_mdays(self.page)
		self.cur_date = {}
		self.cur_date_name = {}
		w += 1 - self._get_config("first_weekday")
		if w < 0:
			w += 7
		for i in range(42):
			if i >= w:
				d += 1
			if d > ld:
				d = 0
				stop = 1
			if d <= 0 or stop == 1:
				out.set("day[%d]"%i, " ")
			else:
				out.set("day[%d]"%i, "%d"%d)
				self.cur_date[i] = self.page[1], self.page[0], d
				
			if d == today:
				out.set("dcol[%d]"%i, self._get_config("today_color"))
			elif tasks.count(d) > 0:
				out.set("dcol[%d]"%i, self._get_config("event_day_color"))
			else:
				out.set("dcol[%d]"%i, self._get_config("day_color"))
		
		out.set("bar1col", self._get_config("bar1_color"))
		out.set("bar2col", self._get_config("bar2_color"))
		self._send_output(out)
		
		return 1

	def call_action(self, action, path, args = []):
		if (action == "next"):
			self.page = self.c.page_increase(self.page)
			self.update_display()
		elif (action == "prev"):
			self.page = self.c.page_decrease(self.page)
			self.update_display()
		elif (action == "cur"):
			self.page = self.c.get_page(time.localtime())
			self.update_display()
		
		elif (action == "events"):
			#if path != []:
			evli = events_list(self.cur_date[self.last_path], self.update_display,
			                   self._get_config("first_weekday"))
			self.hide_events(self.last_path)
			
		elif (action == "enter_date"):
			if path != []:
				self.show_events(path[0])
		elif (action == "leave_date"):
			if path != []:
				self.hide_events(path[0])

	def show_events(self, path):
		self.last_path = path
		try:
			id = self._get_config_id()
			
			did = ""
			for c in id:
				if (c.isdigit()): 
					did += c
				else:
					break
			geo = admin.get_geometry(did)
			x = geo[0]
			y = geo[1] + geo[3]/3
			self.event_views[path] = event_viewer(self.cur_date[path], x, y)
			self.event_views[path].show()
		except KeyError:
			pass

	def hide_events(self, path):
		try:
			self.event_views[path].hide()
		except KeyError:
			pass
	
	def remove_earlier_events(self):
		doc = docdb()
		today = self.c.get_today(self.page)
		i = 0
		orig_page = self.page
		
		if today == 1:
			self.page = self.c.page_decrease(self.page)
			today = self.c.get_today(self.page)
		
		today -= 1
		tasks = self.c.get_days_with_tasks(self.page)
		
		task_number_today = tasks.count(today)
		
		for cv in range(task_number_today):
			t1 = self.c.get_tasks_on_day(today, self.page)[i]
			if t1.has_recurrence == 0:
				# Delete it
				doc.remove_event(t1.ID)
				print "Deleted event " + str(t1.ID) + "at " + time.strftime("%H:%M")
			#else:
			#	ydt = self.c.time.strftime("%Y") + "-" str(eval(self.c.time.strftime("%m"))-1) + self.c.time.strftime("-%d
			#	if t1.recurrence_until == ydt:
			#		# Delete it
			#		doc.remove_event(t1.ID)
			
			i += 1
		
		self.page = orig_page
	
	
def new_sensor(args): 
	return apply(WeeklyCalendar, args)
